/************/
/* INCLUDES */
/************/
#include <plib.h>
#include "stopwatch_prototypes.h"
#include "NU32.h"

/*************/
/* CONSTANTS */
/*************/
#define OVERFLOW_TICKS (4000000000u)  /* the 'u' specifies unsigned */
#define NS_PER_CORE_TICK (25.0)


/********/
/* ISRs */
/********/


/* PLACE DEFINITION OF CORE TIMER ISR HERE */
/* Your ISR must do the following: */
/*     1) reset the value of the core timer to 0 */
/*     2) refill in the value of the _CP0_SET_COMPARE register */
/*     3) call the rollover_handler function correctly */
/*     4) clear the interrupt flag */





/* PLACE THE ISR FOR THE CN MODULE HERE. */
/* ISR must do the following: */
/*     1) immediately store the current state of the core timer and the */
/*         rollover count */
/*     2) determine which button was pressed and determine if it was a falling */
/*         edge (each button triggers two interrupts when pressed, once for */
/*         pushing it down and once for letting it go, since each button is */
/*         wired with an external pull-up we only count the falling edges) */
/*     3) handle the logic for the button presses that was described in the */
/*         assignment.  to keep this ISR short we put this in a function. */
/*     4) be sure to read the correct PORTs and clear the ISR flag!  */
/*     5) you may also want to toggle LEDs to show that the ISR was triggered */



/************************/
/* FUNCTION DEFINITIONS */
/************************/



/* Here is where you put your function for handling the system logic. */
/* The logic should follow that outlined in the assignment.  Note that you will */
/* likely want to use the "static" modifier so that you can have a variable */
/* that is persistent across calls to this function.  If you are unsure what */
/* this means, consult the Crash Course in C document.  You should */
/* also have a prototype for this function in the stopwatch_prototypes.h */



/* this is the function for initializing the CN module on the correct pins */
void initializeCN(void)
{
  /* We are going to be using the USER button for reset, and pin D4
   * (CN13) for start/stop.  So, be sure to wire up pin D4 to one of
   * your buttons.  It is recommended to wire it up with 3.3V and a
   * pull-up resistor.  Just like the USER button on the NU32
   * board. */

  /* Set the TRIS bits correctly for the reset button */
   ??????????
    
  /* Enable CN module */
  CNCONSET = ?????;

  /* Enable the CN1 pin and the CN12 pin*/
  CNENSET = ?????;

  /* Clear any mismatches that may already be there by reading the
   * correct ports*/
  PORTC;
  PORTD;

  /* Set interrupt priority to be 6 with a subpriority of 0*/
  IPC6SET = ?????;
  IPC6CLR = ?????;

  /* Clear interrupt flag */
  IFS1CLR = ?????;

  /* Enable CN interrupts */
  IEC1SET = ?????;

  /* NOTE: we don't have to disable interrupts because we call this
   * function before NU32_Initialize which is what enables them */
}


/* This is a function for starting up timing. It takes an unsigned
 * int and sets the core timer to that value. It also enables core
 * timer interrupts, and sets the _CP0_SET_COMPARE register. */
void start_timing(unsigned int val)
{
  /* enable the interrupt */
  mConfigIntCoreTimer(CT_INT_ON | CT_INT_PRIOR_2);
  /* reset the core count: */
  WriteCoreTimer(val);
  /* initialize compare value: */
  _CP0_SET_COMPARE(OVERFLOW_TICKS);
}

/* This function converts the current CoreTimer count into a double in */
/* units of seconds. */
double get_time_elapsed(unsigned int counts)
{
  return((double) counts * (2.0/((double) SYS_FREQ)));
}

/* This function calculates the total amount of time contributed by */
/* rollovers. */
double get_rollover_time(unsigned int rollovers)
{
  return((double) rollovers*(OVERFLOW_TICKS*(2.0/((double) SYS_FREQ))));
}


/* This function converts the total time as a double in seconds.  The
 * total time is represented as two unsigned integers, one is the
 * number of rollovers, and the other is the current core timer
 * value. This function takes both of these unsigned ints as
 * arguments, and then passes them to other helper functions to
 * calculate the time. */
double calculate_total_time(unsigned int rollovers, unsigned int counts)
{
  return(get_rollover_time(rollovers) + get_time_elapsed(counts));
}


/* This function converts the total time into the correct format, and */
/* the prints it over UART1. */
void display_time(double time)
{
  int hours, minutes, seconds, ms;

  hours = (int) (time/3600.);
  minutes = ((int) (time/60.))%60;
  seconds = ((int) time)%60;
  ms = ((int) (time*1000.))%1000;

  sprintf(NU32_RS232OutBuffer,"Time elapsed:\n\r");
  NU32_WriteUART1(NU32_RS232OutBuffer);
  sprintf(NU32_RS232OutBuffer,"%02d:%02d:%02d.%03d\n\r\n\r",
	  hours, minutes, seconds, ms);
  NU32_WriteUART1(NU32_RS232OutBuffer);

  return;
        
}

/* This function tracks the number of rollovers.  Sending it a value */
/* for inc, increments the count by inc if the reset char is 0.  If */
/* the reset char is 1, we reset the count, and inc is unimportant.
/* If the reset char is 2, we do nothing to the count, we just return
/* its value.  Notice the use of the static modifier! */
unsigned int rollover_handler(unsigned int inc, char reset)
{
  static unsigned int count = 0;
  if (reset == 0)
    count += inc;
  else if (reset == 1)
    count = 0;
  else if (reset == 2)
    return(count);
  else
  {
    count = 0;
    sprintf(NU32_RS232OutBuffer, "[ERROR] rollover_handler\n\r");
    NU32_WriteUART1(NU32_RS232OutBuffer);
  }
  return(count);
}

